/* This file is part of gPHPEdit, a GNOME2 PHP Editor.
 
   Copyright  2003-2004 Andy Jeffries
      andy@gphpedit.org
	  
   For more information or to find the latest release, visit our 
   website at http://www.gphpedit.org/
 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.
 
   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307, USA.
 
   The GNU General Public License is contained in the file COPYING.*/


#include "preferences_dialog.h"
#include "preferences.h"
#include "main_window.h"

#define IS_FONT_NAME(name1, name2) strncmp(name1, name2, MIN(strlen(name1), strlen(name2))) == 0

PreferencesDialog preferences_dialog;
Preferences temp_preferences;
gchar *current_highlighting_element;


static int cmp_families (const void *a, const void *b)
{
	const char *a_name = pango_font_family_get_name (*(PangoFontFamily **)a);
	const char *b_name = pango_font_family_get_name (*(PangoFontFamily **)b);
  
	return g_utf8_collate (a_name, b_name);
}

static GList * get_font_names()
{
	PangoFontFamily **families;
	gint n_families, i;
	GList *fonts = NULL;
  
	pango_context_list_families (gtk_widget_get_pango_context (GTK_WIDGET (main_window.window)),
		&families, &n_families);
	qsort (families, n_families, sizeof (PangoFontFamily *), cmp_families);

	for (i=0; i<n_families; i++) {
		const gchar *name = pango_font_family_get_name (families[i]);
		fonts = g_list_append(fonts, (gchar *)name);
    }
	
	return fonts;
}

static GList *get_font_sizes()
{
	GList *sizes = NULL;
	
	// Again, there's gotta be a nicer way to do this!!!!  AJ 2004-02-09
	
	sizes = g_list_append(sizes, "6");
	sizes = g_list_append(sizes, "8");
	sizes = g_list_append(sizes, "10");
	sizes = g_list_append(sizes, "12");
	sizes = g_list_append(sizes, "14");
	sizes = g_list_append(sizes, "16");
	sizes = g_list_append(sizes, "18");
	sizes = g_list_append(sizes, "20");
	sizes = g_list_append(sizes, "22");
	sizes = g_list_append(sizes, "24");
	sizes = g_list_append(sizes, "26");
	sizes = g_list_append(sizes, "32");
	sizes = g_list_append(sizes, "48");
	sizes = g_list_append(sizes, "72");
	
	return sizes;
}


void get_current_preferences(void)
{
	GList *highlighting_list = NULL;

	// Copy the global preferences struct in to a local copy for editing (in case they click cancel)
	memcpy(&temp_preferences, &preferences, sizeof(preferences));

	// Font drop down elements : General
	highlighting_list = g_list_append(highlighting_list, "Default");
	highlighting_list = g_list_append(highlighting_list, "Line Number (margin)");

	// Font drop down elements : HTML
	highlighting_list = g_list_append(highlighting_list, "HTML Tag");
	highlighting_list = g_list_append(highlighting_list, "HTML Unknown Tag");
	highlighting_list = g_list_append(highlighting_list, "HTML Attribute");
	highlighting_list = g_list_append(highlighting_list, "HTML Unknown Attribute");
	highlighting_list = g_list_append(highlighting_list, "HTML Number");
	highlighting_list = g_list_append(highlighting_list, "HTML Single-quoted String");
	highlighting_list = g_list_append(highlighting_list, "HTML Double-quoted String");
	highlighting_list = g_list_append(highlighting_list, "HTML Comment");
	highlighting_list = g_list_append(highlighting_list, "HTML Entity");
	highlighting_list = g_list_append(highlighting_list, "HTML Script");
	highlighting_list = g_list_append(highlighting_list, "HTML Question");
	highlighting_list = g_list_append(highlighting_list, "HTML Value");

	// Font drop down elements : JavaScript
	highlighting_list = g_list_append(highlighting_list, "JavaScript Multiple-line Comment");
	highlighting_list = g_list_append(highlighting_list, "JavaScript Single-line Comment");
	highlighting_list = g_list_append(highlighting_list, "JavaScript Document Comment");
	highlighting_list = g_list_append(highlighting_list, "JavaScript Word");
	highlighting_list = g_list_append(highlighting_list, "JavaScript Keyword");
	highlighting_list = g_list_append(highlighting_list, "JavaScript Single-quoted String");
	highlighting_list = g_list_append(highlighting_list, "JavaScript Double-quoted String");
	highlighting_list = g_list_append(highlighting_list, "JavaScript Symbol");

	// Font drop down elements : PHP
	highlighting_list = g_list_append(highlighting_list, "PHP Default");
	highlighting_list = g_list_append(highlighting_list, "PHP 'HString'");
	highlighting_list = g_list_append(highlighting_list, "PHP Simple String");
	highlighting_list = g_list_append(highlighting_list, "PHP Word");
	highlighting_list = g_list_append(highlighting_list, "PHP Number");
	highlighting_list = g_list_append(highlighting_list, "PHP Variable");
	highlighting_list = g_list_append(highlighting_list, "PHP Single-line Comment");
	highlighting_list = g_list_append(highlighting_list, "PHP Multiple-line Comment");

	// Font drop down elements : CSS
	highlighting_list = g_list_append(highlighting_list, "CSS Tag");
	highlighting_list = g_list_append(highlighting_list, "CSS Class");
	highlighting_list = g_list_append(highlighting_list, "CSS Psuedoclass");
	highlighting_list = g_list_append(highlighting_list, "CSS Unknown Pseudoclass");
	highlighting_list = g_list_append(highlighting_list, "CSS Operator");
	highlighting_list = g_list_append(highlighting_list, "CSS Identifier");
	highlighting_list = g_list_append(highlighting_list, "CSS Unknown Identifier");
	highlighting_list = g_list_append(highlighting_list, "CSS Value");
	highlighting_list = g_list_append(highlighting_list, "CSS Comment");
	highlighting_list = g_list_append(highlighting_list, "CSS ID");
	highlighting_list = g_list_append(highlighting_list, "CSS Important");
	highlighting_list = g_list_append(highlighting_list, "CSS Directive");

	// Font drop down elements : SQL
	highlighting_list = g_list_append(highlighting_list, "SQL Word");
	highlighting_list = g_list_append(highlighting_list, "SQL Identifier");
	highlighting_list = g_list_append(highlighting_list, "SQL Number");
	highlighting_list = g_list_append(highlighting_list, "SQL String");
	highlighting_list = g_list_append(highlighting_list, "SQL Operator");
	highlighting_list = g_list_append(highlighting_list, "SQL Comment");

	preferences_dialog.combo1_items = highlighting_list;
}


void set_current_preferences(void)
{
	// Copy the local copy back to the global list
	memcpy(&preferences, &temp_preferences, sizeof(preferences));

	// Save the preferences
	preferences_save();
}

void set_controls_to_highlight(gchar *setting_name, gchar *fontname, gint fontsize, gboolean bold, gboolean italic, gint fore, gint back)
{
	gchar *sfontsize;
	
	// Debug print for preferences being set
	// g_print("Getting %s: %s %d %d %d %d %d\n", setting_name, fontname, fontsize, bold, italic, fore, back);
	
	preferences_dialog.changing_highlight_element=TRUE;
	
	fontname++; // Ignore the initial ! for Pango rendering
	
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(preferences_dialog.combo_fontlist)->entry), fontname);
	sfontsize = g_strdup_printf("%d", fontsize);
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(preferences_dialog.combo_fontsize)->entry), sfontsize);
	g_free(sfontsize);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(preferences_dialog.bold_button), bold);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(preferences_dialog.italic_button), italic);
	
	gnome_color_picker_set_i8(GNOME_COLOR_PICKER(preferences_dialog.fore_color_picker), 
		(fore & 0xff), (fore & 0xff00) >> 8, (fore & 0xff0000) >> 16, 0);
	
	gnome_color_picker_set_i8(GNOME_COLOR_PICKER(preferences_dialog.back_color_picker), 
		(back & 0xff), (back & 0xff00) >> 8, (back & 0xff0000) >> 16, 0);
	preferences_dialog.changing_highlight_element=FALSE;
}

void get_current_highlighting_settings(gchar *name)
{
	if (strlen(name)==0) {
		return;
	}

	if (IS_FONT_NAME(current_highlighting_element, "Default")) {
		set_controls_to_highlight("Default", temp_preferences.default_font, temp_preferences.default_size, 
			temp_preferences.default_bold, temp_preferences.default_italic,
			temp_preferences.default_fore, temp_preferences.default_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "Line Number (margin)")) {
		set_controls_to_highlight("Line Number (margin)", temp_preferences.line_number_font, temp_preferences.line_number_size, 
			temp_preferences.line_number_bold, temp_preferences.line_number_italic,
			temp_preferences.line_number_fore, temp_preferences.line_number_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Tag")) {
		set_controls_to_highlight("HTML Tag", temp_preferences.html_tag_font, temp_preferences.html_tag_size, 
			temp_preferences.html_tag_bold, temp_preferences.html_tag_italic,
			temp_preferences.html_tag_fore, temp_preferences.html_tag_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Unknown Tag")) {
		set_controls_to_highlight("HTML Unknown Tag", temp_preferences.html_tag_unknown_font, temp_preferences.html_tag_unknown_size, 
			temp_preferences.html_tag_unknown_bold, temp_preferences.html_tag_unknown_italic,
			temp_preferences.html_tag_unknown_fore, temp_preferences.html_tag_unknown_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Attribute")) {
		set_controls_to_highlight("HTML Attribute", temp_preferences.html_attribute_font, temp_preferences.html_attribute_size, 
			temp_preferences.html_attribute_bold, temp_preferences.html_attribute_italic,
			temp_preferences.html_attribute_fore, temp_preferences.html_attribute_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Unknown Attribute")) {
		set_controls_to_highlight("HTML Unknown Attribute", temp_preferences.html_attribute_unknown_font, temp_preferences.html_attribute_unknown_size, 
			temp_preferences.html_attribute_unknown_bold, temp_preferences.html_attribute_unknown_italic,
			temp_preferences.html_attribute_unknown_fore, temp_preferences.html_attribute_unknown_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Number")) {
		set_controls_to_highlight("HTML Number", temp_preferences.html_number_font, temp_preferences.html_number_size, 
			temp_preferences.html_number_bold, temp_preferences.html_number_italic,
			temp_preferences.html_number_fore, temp_preferences.html_number_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Single-quoted String")) {
		set_controls_to_highlight("HTML Single-quoted String", temp_preferences.html_single_string_font, temp_preferences.html_single_string_size, 
			temp_preferences.html_single_string_bold, temp_preferences.html_single_string_italic,
			temp_preferences.html_single_string_fore, temp_preferences.html_single_string_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Double-quoted String")) {
		set_controls_to_highlight("HTML Double-quoted String", temp_preferences.html_double_string_font, temp_preferences.html_double_string_size, 
			temp_preferences.html_double_string_bold, temp_preferences.html_double_string_italic,
			temp_preferences.html_double_string_fore, temp_preferences.html_double_string_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Comment")) {
		set_controls_to_highlight("HTML Comment", temp_preferences.html_comment_font, temp_preferences.html_comment_size, 
			temp_preferences.html_comment_bold, temp_preferences.html_comment_italic,
			temp_preferences.html_comment_fore, temp_preferences.html_comment_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Entity")) {
		set_controls_to_highlight("HTML Entity", temp_preferences.html_entity_font, temp_preferences.html_entity_size, 
			temp_preferences.html_entity_bold , temp_preferences.html_entity_italic,
			temp_preferences.html_entity_fore, temp_preferences.html_entity_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Script")) {
		set_controls_to_highlight("HTML Script", temp_preferences.html_script_font, temp_preferences.html_script_size, 
			temp_preferences.html_script_bold, temp_preferences.html_script_italic,
			temp_preferences.html_script_fore, temp_preferences.html_script_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Question")) {
		set_controls_to_highlight("HTML Question", temp_preferences.html_question_font, temp_preferences.html_question_size, 
			temp_preferences.html_question_bold, temp_preferences.html_question_italic,
			temp_preferences.html_question_fore, temp_preferences.html_question_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Value")) {
		set_controls_to_highlight("HTML Value", temp_preferences.html_value_font, temp_preferences.html_value_size, 
			temp_preferences.html_value_bold, temp_preferences.html_value_italic,
			temp_preferences.html_value_fore, temp_preferences.html_value_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "JavaScript Multiple-line Comment")) {
		set_controls_to_highlight("JavaScript Multiple-line Comment", temp_preferences.javascript_comment_font, temp_preferences.javascript_comment_size, 
			temp_preferences.javascript_comment_bold, temp_preferences.javascript_comment_italic,
			temp_preferences.javascript_comment_fore, temp_preferences.javascript_comment_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "JavaScript Single-line Comment")) {
		set_controls_to_highlight("JavaScript Single-line Comment", temp_preferences.javascript_comment_line_font, temp_preferences.javascript_comment_line_size, 
			temp_preferences.javascript_comment_line_bold, temp_preferences.javascript_comment_line_italic,
			temp_preferences.javascript_comment_line_fore, temp_preferences.javascript_comment_line_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "JavaScript Document Comment")) {
		set_controls_to_highlight("JavaScript Document Comment", temp_preferences.javascript_comment_doc_font, temp_preferences.javascript_comment_doc_size, 
			temp_preferences.javascript_comment_doc_bold, temp_preferences.javascript_comment_doc_italic,
			temp_preferences.javascript_comment_doc_fore, temp_preferences.javascript_comment_doc_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "JavaScript Word")) {
		set_controls_to_highlight("JavaScript Word", temp_preferences.javascript_word_font, temp_preferences.javascript_word_size, 
			temp_preferences.javascript_word_bold, temp_preferences.javascript_word_italic,
			temp_preferences.javascript_word_fore, temp_preferences.javascript_word_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "JavaScript Keyword")) {
		set_controls_to_highlight("JavaScript Keyword", temp_preferences.javascript_keyword_font, temp_preferences.javascript_keyword_size, 
			temp_preferences.javascript_keyword_bold, temp_preferences.javascript_keyword_italic,
			temp_preferences.javascript_keyword_fore, temp_preferences.javascript_keyword_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "JavaScript Single-quoted String")) {
		set_controls_to_highlight("JavaScript Single-quoted String", temp_preferences.javascript_doublestring_font, temp_preferences.javascript_doublestring_size, 
			temp_preferences.javascript_doublestring_bold, temp_preferences.javascript_doublestring_italic,
			temp_preferences.javascript_doublestring_fore, temp_preferences.javascript_doublestring_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "JavaScript Double-quoted String")) {
		set_controls_to_highlight("JavaScript Double-quoted String", temp_preferences.javascript_singlestring_font, temp_preferences.javascript_singlestring_size, 
			temp_preferences.javascript_singlestring_bold, temp_preferences.javascript_singlestring_italic,
			temp_preferences.javascript_singlestring_fore, temp_preferences.javascript_singlestring_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "JavaScript Symbol")) {
		set_controls_to_highlight("JavaScript Symbol", temp_preferences.javascript_symbols_font, temp_preferences.javascript_symbols_size, 
			temp_preferences.javascript_symbols_bold, temp_preferences.javascript_symbols_italic,
			temp_preferences.javascript_symbols_fore, temp_preferences.javascript_symbols_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "PHP Default")) {
		set_controls_to_highlight("PHP Default", temp_preferences.php_default_font, temp_preferences.php_default_size, 
			temp_preferences.php_default_bold, temp_preferences.php_default_italic,
			temp_preferences.php_default_fore, temp_preferences.php_default_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "PHP 'HString'")) {
		set_controls_to_highlight("PHP 'HString'", temp_preferences.php_hstring_font, temp_preferences.php_hstring_size, 
			temp_preferences.php_hstring_bold, temp_preferences.php_hstring_italic,
			temp_preferences.php_hstring_fore, temp_preferences.php_hstring_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "PHP Simple String")) {
		set_controls_to_highlight("PHP Simple String", temp_preferences.php_simplestring_font, temp_preferences.php_simplestring_size, 
			temp_preferences.php_simplestring_bold, temp_preferences.php_simplestring_italic,
			temp_preferences.php_simplestring_fore, temp_preferences.php_simplestring_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "PHP Word")) {
		set_controls_to_highlight("PHP Word", temp_preferences.php_word_font, temp_preferences.php_word_size, 
			temp_preferences.php_word_bold, temp_preferences.php_word_italic,
			temp_preferences.php_word_fore, temp_preferences.php_word_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "PHP Number")) {
		set_controls_to_highlight("PHP Number", temp_preferences.php_number_font, temp_preferences.php_number_size, 
			temp_preferences.php_number_bold, temp_preferences.php_number_italic,
			temp_preferences.php_number_fore, temp_preferences.php_number_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "PHP Variable")) {
		set_controls_to_highlight("PHP Variable", temp_preferences.php_variable_font, temp_preferences.php_variable_size, 
			temp_preferences.php_variable_bold, temp_preferences.php_variable_italic,
			temp_preferences.php_variable_fore, temp_preferences.php_variable_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "PHP Single-line Comment")) {
		set_controls_to_highlight("PHP Single-line Comment", temp_preferences.php_comment_font, temp_preferences.php_comment_size, 
			temp_preferences.php_comment_bold, temp_preferences.php_comment_italic,
			temp_preferences.php_comment_fore, temp_preferences.php_comment_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "PHP Multiple-line Comment")) {
		set_controls_to_highlight("PHP Multiple-line Comment", temp_preferences.php_comment_line_font, temp_preferences.php_comment_line_size, 
			temp_preferences.php_comment_line_bold, temp_preferences.php_comment_line_italic,
			temp_preferences.php_comment_line_fore, temp_preferences.php_comment_line_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Tag")) {
		set_controls_to_highlight("CSS Tag", temp_preferences.css_tag_font, temp_preferences.css_tag_size, 
			temp_preferences.css_tag_bold, temp_preferences.css_tag_italic,
			temp_preferences.css_tag_fore, temp_preferences.css_tag_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Class")) {
		set_controls_to_highlight("CSS Class", temp_preferences.css_class_font, temp_preferences.css_class_size, 
			temp_preferences.css_class_bold, temp_preferences.css_class_italic,
			temp_preferences.css_class_fore, temp_preferences.css_class_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Psuedoclass")) {
		set_controls_to_highlight("CSS Psuedoclass", temp_preferences.css_pseudoclass_font, temp_preferences.css_pseudoclass_size, 
			temp_preferences.css_pseudoclass_bold, temp_preferences.css_pseudoclass_italic,
			temp_preferences.css_pseudoclass_fore, temp_preferences.css_pseudoclass_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Unknown Pseudoclass")) {
		set_controls_to_highlight("CSS Unknown Pseudoclass", temp_preferences.css_unknown_pseudoclass_font, temp_preferences.css_unknown_pseudoclass_size, 
			temp_preferences.css_unknown_pseudoclass_bold, temp_preferences.css_unknown_pseudoclass_italic,
			temp_preferences.css_unknown_pseudoclass_fore, temp_preferences.css_unknown_pseudoclass_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Operator")) {
		set_controls_to_highlight("CSS Operator", temp_preferences.css_operator_font, temp_preferences.css_operator_size, 
			temp_preferences.css_operator_bold, temp_preferences.css_operator_italic,
			temp_preferences.css_operator_fore, temp_preferences.css_operator_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Identifier")) {
		set_controls_to_highlight("CSS Identifier", temp_preferences.css_identifier_font, temp_preferences.css_identifier_size, 
			temp_preferences.css_identifier_bold, temp_preferences.css_identifier_italic,
			temp_preferences.css_identifier_fore, temp_preferences.css_identifier_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Unknown Identifier")) {
		set_controls_to_highlight("CSS Unknown Identifier", temp_preferences.css_unknown_identifier_font, temp_preferences.css_unknown_identifier_size, 
			temp_preferences.css_unknown_identifier_bold, temp_preferences.css_unknown_identifier_italic,
			temp_preferences.css_unknown_identifier_fore, temp_preferences.css_unknown_identifier_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Value")) {
		set_controls_to_highlight("CSS Value", temp_preferences.css_value_font, temp_preferences.css_value_size, 
			temp_preferences.css_value_bold, temp_preferences.css_value_italic,
			temp_preferences.css_value_fore, temp_preferences.css_value_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Comment")) {
		set_controls_to_highlight("CSS Comment", temp_preferences.css_comment_font, temp_preferences.css_comment_size, 
			temp_preferences.css_comment_bold, temp_preferences.css_comment_italic,
			temp_preferences.css_comment_fore, temp_preferences.css_comment_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS ID")) {
		set_controls_to_highlight("CSS ID", temp_preferences.css_id_font, temp_preferences.css_id_size, 
			temp_preferences.css_id_bold, temp_preferences.css_id_italic,
			temp_preferences.css_id_fore, temp_preferences.css_id_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Important")) {
		set_controls_to_highlight("CSS Important", temp_preferences.css_important_font, temp_preferences.css_important_size, 
			temp_preferences.css_important_bold, temp_preferences.css_important_italic,
			temp_preferences.css_important_fore, temp_preferences.css_important_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Directive")) {
		set_controls_to_highlight("CSS Directive", temp_preferences.css_directive_font, temp_preferences.css_directive_size, 
			temp_preferences.css_directive_bold, temp_preferences.css_directive_italic,
			temp_preferences.css_directive_fore, temp_preferences.css_directive_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "SQL Word")) {
		set_controls_to_highlight("SQL Word", temp_preferences.sql_word_font, temp_preferences.sql_word_size, 
			temp_preferences.sql_word_bold, temp_preferences.sql_word_italic,
			temp_preferences.sql_word_fore, temp_preferences.sql_word_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "SQL Identifier")) {
		set_controls_to_highlight("SQL Identifier", temp_preferences.sql_identifier_font, temp_preferences.sql_identifier_size, 
			temp_preferences.sql_identifier_bold, temp_preferences.sql_identifier_italic,
			temp_preferences.sql_identifier_fore, temp_preferences.sql_identifier_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "SQL Number")) {
		set_controls_to_highlight("SQL Number", temp_preferences.sql_number_font, temp_preferences.sql_number_size, 
			temp_preferences.sql_number_bold, temp_preferences.sql_number_italic,
			temp_preferences.sql_number_fore, temp_preferences.sql_number_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "SQL String")) {
		set_controls_to_highlight("SQL String", temp_preferences.sql_string_font, temp_preferences.sql_string_size, 
			temp_preferences.sql_string_bold, temp_preferences.sql_string_italic,
			temp_preferences.sql_string_fore, temp_preferences.sql_string_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "SQL Operator")) {
		set_controls_to_highlight("SQL Operator", temp_preferences.sql_operator_font, temp_preferences.sql_operator_size, 
			temp_preferences.sql_operator_bold, temp_preferences.sql_operator_italic,
			temp_preferences.sql_operator_fore, temp_preferences.sql_operator_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "SQL Comment")) {
		set_controls_to_highlight("SQL Comment", temp_preferences.sql_comment_font, temp_preferences.sql_comment_size, 
			temp_preferences.sql_comment_bold, temp_preferences.sql_comment_italic,
			temp_preferences.sql_comment_fore, temp_preferences.sql_comment_back);
	}
}


void on_combo_entry_changed(GtkEntry *entry, gpointer data)
{
	current_highlighting_element = (gchar *)gtk_entry_get_text(GTK_ENTRY(entry));
	get_current_highlighting_settings(current_highlighting_element);
}


void ok_clicked(GtkButton *button, gpointer data)
{
	set_current_preferences();
	//g_print("OK\n");
	gtk_widget_destroy(preferences_dialog.window);
}


void get_control_values_to_highlight(gchar *setting_name, gchar **fontname, gint *fontsize, gboolean *bold, gboolean *italic, gint *fore, gint *back)
{
	GString *tempfontname;
	guint8 red, blue, green, alpha;
	
	if (preferences_dialog.changing_highlight_element) {
		return;
	}
	
	tempfontname = g_string_new(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(preferences_dialog.combo_fontlist)->entry)));
	tempfontname = g_string_prepend(tempfontname, "!");
		
	*fontname = tempfontname->str;
	g_string_free(tempfontname, FALSE);
	*fontsize = atoi(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(preferences_dialog.combo_fontsize)->entry)));
	*bold = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(preferences_dialog.bold_button));
	*italic = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(preferences_dialog.italic_button));
	
	gnome_color_picker_get_i8(GNOME_COLOR_PICKER(preferences_dialog.fore_color_picker), &red, &blue, &green, &alpha);
	*fore = (green<<16) | (blue<<8) | red;
	
	gnome_color_picker_get_i8(GNOME_COLOR_PICKER(preferences_dialog.back_color_picker), &red, &blue, &green, &alpha);
	*back = (green<<16) | (blue<<8) | red;

	// Debug print for preferences being set
	// g_print("Setting %s: %s %d %d %d %d %d\n", setting_name, *fontname, *fontsize, *bold, *italic, *fore, *back);	
}

void set_current_highlighting_font()
{
	if (IS_FONT_NAME(current_highlighting_element, "Default")) {
		get_control_values_to_highlight("Default", &temp_preferences.default_font, &temp_preferences.default_size,
			&temp_preferences.default_bold, &temp_preferences.default_italic,
			&temp_preferences.default_fore, &temp_preferences.default_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "Line Number (margin)")) {
		get_control_values_to_highlight("Line Number (margin)", &temp_preferences.line_number_font, &temp_preferences.line_number_size,
			&temp_preferences.line_number_bold, &temp_preferences.line_number_italic,
			&temp_preferences.line_number_fore, &temp_preferences.line_number_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Tag")) {
		get_control_values_to_highlight("HTML Tag", &temp_preferences.html_tag_font, &temp_preferences.html_tag_size,
			&temp_preferences.html_tag_bold, &temp_preferences.html_tag_italic,
			&temp_preferences.html_tag_fore, &temp_preferences.html_tag_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Unknown Tag")) {
		get_control_values_to_highlight("HTML Unknown Tag", &temp_preferences.html_tag_unknown_font, &temp_preferences.html_tag_unknown_size,
			&temp_preferences.html_tag_unknown_bold, &temp_preferences.html_tag_unknown_italic,
			&temp_preferences.html_tag_unknown_fore, &temp_preferences.html_tag_unknown_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Attribute")) {
		get_control_values_to_highlight("HTML Attribute", &temp_preferences.html_attribute_font, &temp_preferences.html_attribute_size,
			&temp_preferences.html_attribute_bold, &temp_preferences.html_attribute_italic,
			&temp_preferences.html_attribute_fore, &temp_preferences.html_attribute_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Unknown Attribute")) {
		get_control_values_to_highlight("HTML Unknown Attribute", &temp_preferences.html_attribute_unknown_font, &temp_preferences.html_attribute_unknown_size,
			&temp_preferences.html_attribute_unknown_bold, &temp_preferences.html_attribute_unknown_italic,
			&temp_preferences.html_attribute_unknown_fore, &temp_preferences.html_attribute_unknown_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Number")) {
		get_control_values_to_highlight("HTML Number", &temp_preferences.html_number_font, &temp_preferences.html_number_size,
			&temp_preferences.html_number_bold, &temp_preferences.html_number_italic,
			&temp_preferences.html_number_fore, &temp_preferences.html_number_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Single-quoted String")) {
		get_control_values_to_highlight("HTML Single-quoted String", &temp_preferences.html_single_string_font, &temp_preferences.html_single_string_size,
			&temp_preferences.html_single_string_bold, &temp_preferences.html_single_string_italic,
			&temp_preferences.html_single_string_fore, &temp_preferences.html_single_string_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Double-quoted String")) {
		get_control_values_to_highlight("HTML Double-quoted String", &temp_preferences.html_double_string_font, &temp_preferences.html_double_string_size,
			&temp_preferences.html_double_string_italic, &temp_preferences.html_double_string_italic,
			&temp_preferences.html_double_string_fore, &temp_preferences.html_double_string_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Comment")) {
		get_control_values_to_highlight("HTML Comment", &temp_preferences.html_comment_font, &temp_preferences.html_comment_size,
			&temp_preferences.html_comment_bold, &temp_preferences.html_comment_italic,
			&temp_preferences.html_comment_fore, &temp_preferences.html_comment_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Entity")) {
		get_control_values_to_highlight("HTML Entity", &temp_preferences.html_entity_font, &temp_preferences.html_entity_size,
			&temp_preferences.html_entity_bold, &temp_preferences.html_entity_italic,
			&temp_preferences.html_entity_fore, &temp_preferences.html_entity_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Script")) {
		get_control_values_to_highlight("HTML Script", &temp_preferences.html_script_font, &temp_preferences.html_script_size,
			&temp_preferences.html_script_bold, &temp_preferences.html_script_italic,
			&temp_preferences.html_script_fore, &temp_preferences.html_script_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Question")) {
		get_control_values_to_highlight("HTML Question", &temp_preferences.html_question_font, &temp_preferences.html_question_size,
			&temp_preferences.html_question_bold, &temp_preferences.html_question_italic,
			&temp_preferences.html_question_fore, &temp_preferences.html_question_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "HTML Value")) {
		get_control_values_to_highlight("HTML Value", &temp_preferences.html_value_font, &temp_preferences.html_value_size,
			&temp_preferences.html_value_bold, &temp_preferences.html_value_italic,
			&temp_preferences.html_value_fore, &temp_preferences.html_value_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "JavaScript Multiple-line Comment")) {
		get_control_values_to_highlight("JavaScript Multiple-line Comment", &temp_preferences.javascript_comment_font, &temp_preferences.javascript_comment_size,
			&temp_preferences.javascript_comment_bold, &temp_preferences.javascript_comment_italic,
			&temp_preferences.javascript_comment_fore, &temp_preferences.javascript_comment_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "JavaScript Single-line Comment")) {
		get_control_values_to_highlight("JavaScript Single-line Comment", &temp_preferences.javascript_comment_line_font, &temp_preferences.javascript_comment_line_size,
			&temp_preferences.javascript_comment_line_bold, &temp_preferences.javascript_comment_line_italic,
			&temp_preferences.javascript_comment_line_fore, &temp_preferences.javascript_comment_line_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "JavaScript Document Comment")) {
		get_control_values_to_highlight("JavaScript Document Comment", &temp_preferences.javascript_comment_doc_font, &temp_preferences.javascript_comment_doc_size,
			&temp_preferences.javascript_comment_doc_bold, &temp_preferences.javascript_comment_doc_italic,
			&temp_preferences.javascript_comment_doc_fore, &temp_preferences.javascript_comment_doc_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "JavaScript Word")) {
		get_control_values_to_highlight("JavaScript Word", &temp_preferences.javascript_word_font, &temp_preferences.javascript_word_size,
			&temp_preferences.javascript_word_bold, &temp_preferences.javascript_word_italic,
			&temp_preferences.javascript_word_fore, &temp_preferences.javascript_word_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "JavaScript Keyword")) {
		get_control_values_to_highlight("JavaScript Keyword", &temp_preferences.javascript_keyword_font, &temp_preferences.javascript_keyword_size,
			&temp_preferences.javascript_keyword_bold, &temp_preferences.javascript_keyword_italic,
			&temp_preferences.javascript_keyword_fore, &temp_preferences.javascript_keyword_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "JavaScript Single-quoted String")) {
		get_control_values_to_highlight("JavaScript Single-quoted String", &temp_preferences.javascript_doublestring_font, &temp_preferences.javascript_doublestring_size,
			&temp_preferences.javascript_doublestring_bold, &temp_preferences.javascript_doublestring_italic,
			&temp_preferences.javascript_doublestring_fore, &temp_preferences.javascript_doublestring_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "JavaScript Double-quoted String")) {
		get_control_values_to_highlight("JavaScript Double-quoted String", &temp_preferences.javascript_singlestring_font, &temp_preferences.javascript_singlestring_size,
			&temp_preferences.javascript_singlestring_bold, &temp_preferences.javascript_singlestring_italic,
			&temp_preferences.javascript_singlestring_fore, &temp_preferences.javascript_singlestring_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "JavaScript Symbol")) {
		get_control_values_to_highlight("JavaScript Symbol", &temp_preferences.javascript_symbols_font, &temp_preferences.javascript_symbols_size,
			&temp_preferences.javascript_symbols_bold, &temp_preferences.default_italic,
			&temp_preferences.javascript_symbols_fore, &temp_preferences.javascript_symbols_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "PHP Default")) {
		get_control_values_to_highlight("PHP Default", &temp_preferences.php_default_font, &temp_preferences.php_default_size,
			&temp_preferences.php_default_bold, &temp_preferences.php_default_italic,
			&temp_preferences.php_default_fore, &temp_preferences.php_default_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "PHP 'HString'")) {
		get_control_values_to_highlight("PHP 'HString'", &temp_preferences.php_hstring_font, &temp_preferences.php_hstring_size,
			&temp_preferences.php_hstring_bold, &temp_preferences.php_hstring_italic,
			&temp_preferences.php_hstring_fore, &temp_preferences.php_hstring_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "PHP Simple String")) {
		get_control_values_to_highlight("PHP Simple String", &temp_preferences.php_simplestring_font, &temp_preferences.php_simplestring_size,
			&temp_preferences.php_simplestring_bold, &temp_preferences.php_simplestring_italic,
			&temp_preferences.php_simplestring_fore, &temp_preferences.php_simplestring_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "PHP Word")) {
		get_control_values_to_highlight("PHP Word", &temp_preferences.php_word_font, &temp_preferences.php_word_size,
			&temp_preferences.php_word_bold, &temp_preferences.php_word_italic,
			&temp_preferences.php_word_fore, &temp_preferences.php_word_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "PHP Number")) {
		get_control_values_to_highlight("PHP Number", &temp_preferences.php_number_font, &temp_preferences.php_number_size,
			&temp_preferences.php_number_bold, &temp_preferences.php_number_italic,
			&temp_preferences.php_number_fore, &temp_preferences.php_number_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "PHP Variable")) {
		get_control_values_to_highlight("PHP Variable", &temp_preferences.php_variable_font, &temp_preferences.php_variable_size,
			&temp_preferences.php_variable_bold, &temp_preferences.php_variable_italic,
			&temp_preferences.php_variable_fore, &temp_preferences.php_variable_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "PHP Single-line Comment")) {
		get_control_values_to_highlight("PHP Single-line Comment", &temp_preferences.php_comment_font, &temp_preferences.php_comment_size,
			&temp_preferences.php_comment_bold, &temp_preferences.php_comment_italic,
			&temp_preferences.php_comment_fore, &temp_preferences.php_comment_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "PHP Multiple-line Comment")) {
		get_control_values_to_highlight("PHP Multiple-line Comment", &temp_preferences.php_comment_line_font, &temp_preferences.php_comment_line_size,
			&temp_preferences.php_comment_line_bold, &temp_preferences.php_comment_line_italic,
			&temp_preferences.php_comment_line_fore, &temp_preferences.php_comment_line_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Tag")) {
		get_control_values_to_highlight("CSS Tag", &temp_preferences.css_tag_font, &temp_preferences.css_tag_size,
			&temp_preferences.css_tag_bold, &temp_preferences.css_tag_italic,
			&temp_preferences.css_tag_fore, &temp_preferences.css_tag_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Class")) {
		get_control_values_to_highlight("CSS Class", &temp_preferences.css_class_font, &temp_preferences.css_class_size,
			&temp_preferences.css_class_bold, &temp_preferences.css_class_italic,
			&temp_preferences.css_class_fore, &temp_preferences.css_class_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Psuedoclass")) {
		get_control_values_to_highlight("CSS Psuedoclass", &temp_preferences.css_pseudoclass_font, &temp_preferences.css_pseudoclass_size,
			&temp_preferences.css_pseudoclass_bold, &temp_preferences.css_pseudoclass_italic,
			&temp_preferences.css_pseudoclass_fore, &temp_preferences.css_pseudoclass_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Unknown Pseudoclass")) {
		get_control_values_to_highlight("CSS Unknown Pseudoclass", &temp_preferences.css_unknown_pseudoclass_font, &temp_preferences.css_unknown_pseudoclass_size,
			&temp_preferences.css_unknown_pseudoclass_bold, &temp_preferences.css_unknown_pseudoclass_italic,
			&temp_preferences.css_unknown_pseudoclass_fore, &temp_preferences.css_unknown_pseudoclass_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Operator")) {
		get_control_values_to_highlight("CSS Operator", &temp_preferences.css_operator_font, &temp_preferences.css_operator_size,
			&temp_preferences.css_operator_bold, &temp_preferences.css_operator_italic,
			&temp_preferences.css_operator_fore, &temp_preferences.css_operator_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Identifier")) {
		get_control_values_to_highlight("CSS Identifier", &temp_preferences.css_identifier_font, &temp_preferences.css_identifier_size,
			&temp_preferences.css_identifier_bold, &temp_preferences.css_identifier_italic,
			&temp_preferences.css_identifier_fore, &temp_preferences.css_identifier_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Unknown Identifier")) {
		get_control_values_to_highlight("CSS Unknown Identifier", &temp_preferences.css_unknown_identifier_font, &temp_preferences.css_unknown_identifier_size,
			&temp_preferences.css_unknown_identifier_bold, &temp_preferences.css_unknown_identifier_italic,
			&temp_preferences.css_unknown_identifier_fore, &temp_preferences.css_unknown_identifier_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Value")) {
		get_control_values_to_highlight("CSS Value", &temp_preferences.css_value_font, &temp_preferences.css_value_size,
			&temp_preferences.css_value_bold, &temp_preferences.css_value_italic,
			&temp_preferences.css_value_fore, &temp_preferences.css_value_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Comment")) {
		get_control_values_to_highlight("CSS Comment", &temp_preferences.css_comment_font, &temp_preferences.css_comment_size,
			&temp_preferences.css_comment_bold, &temp_preferences.css_comment_italic,
			&temp_preferences.css_comment_fore, &temp_preferences.css_comment_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS ID")) {
		get_control_values_to_highlight("CSS ID", &temp_preferences.css_id_font, &temp_preferences.css_id_size,
			&temp_preferences.css_id_bold, &temp_preferences.css_id_italic,
			&temp_preferences.css_id_fore, &temp_preferences.css_id_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Important")) {
		get_control_values_to_highlight("CSS Important", &temp_preferences.css_important_font, &temp_preferences.css_important_size,
			&temp_preferences.css_important_bold, &temp_preferences.css_important_italic,
			&temp_preferences.css_important_fore, &temp_preferences.css_important_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "CSS Directive")) {
		get_control_values_to_highlight("CSS Directive", &temp_preferences.css_directive_font, &temp_preferences.css_directive_size,
			&temp_preferences.css_directive_bold, &temp_preferences.css_directive_italic,
			&temp_preferences.css_directive_fore, &temp_preferences.css_directive_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "SQL Word")) {
		get_control_values_to_highlight("SQL Word", &temp_preferences.sql_word_font, &temp_preferences.sql_word_size,
			&temp_preferences.sql_word_bold, &temp_preferences.sql_word_italic,
			&temp_preferences.sql_word_fore, &temp_preferences.sql_word_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "SQL Identifier")) {
		get_control_values_to_highlight("SQL Identifier", &temp_preferences.sql_identifier_font, &temp_preferences.sql_identifier_size,
			&temp_preferences.sql_identifier_bold, &temp_preferences.sql_identifier_italic,
			&temp_preferences.sql_identifier_fore, &temp_preferences.sql_identifier_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "SQL Number")) {
		get_control_values_to_highlight("SQL Number", &temp_preferences.sql_number_font, &temp_preferences.sql_number_size,
			&temp_preferences.sql_number_bold, &temp_preferences.sql_number_italic,
			&temp_preferences.sql_number_fore, &temp_preferences.sql_number_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "SQL String")) {
		get_control_values_to_highlight("SQL String", &temp_preferences.sql_string_font, &temp_preferences.sql_string_size,
			&temp_preferences.sql_string_bold, &temp_preferences.sql_string_italic,
			&temp_preferences.sql_string_fore, &temp_preferences.sql_string_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "SQL Operator")) {
		get_control_values_to_highlight("SQL Operator", &temp_preferences.sql_operator_font, &temp_preferences.sql_operator_size,
			&temp_preferences.sql_operator_bold, &temp_preferences.sql_operator_italic,
			&temp_preferences.sql_operator_fore, &temp_preferences.sql_operator_back);
	}
	if (IS_FONT_NAME(current_highlighting_element, "SQL Comment")) {
		get_control_values_to_highlight("SQL Comment", &temp_preferences.sql_comment_font, &temp_preferences.sql_comment_size,
			&temp_preferences.sql_comment_bold, &temp_preferences.sql_comment_italic,
			&temp_preferences.sql_comment_fore, &temp_preferences.sql_comment_back);
	}
}

void on_bold_toggle(GtkToggleButton *togglebutton, gpointer user_data)
{
	set_current_highlighting_font();
}


void on_italic_toggle(GtkToggleButton *togglebutton, gpointer user_data)
{
	set_current_highlighting_font();
}


void on_fontname_entry_changed(GtkEntry *Entry, gpointer data)
{
	set_current_highlighting_font();
}


void on_fontsize_entry_changed(GtkEntry *Entry, gpointer data)
{
	set_current_highlighting_font();
}


void preferences_window_destroyed(GtkWidget *widget, gpointer data)
{
	gtk_grab_remove(preferences_dialog.window);
	preferences_dialog.window = NULL;
}


void on_fore_changed(GnomeColorPicker *cp, guint red, guint green, guint blue, guint alpha, gpointer userdate)
{
	set_current_highlighting_font();
}


void on_back_changed(GnomeColorPicker *cp, guint red, guint green, guint blue, guint alpha, gpointer userdate)
{
	set_current_highlighting_font();
}


void on_edge_colour_changed(GnomeColorPicker *cp, guint red, guint green, guint blue, guint alpha, gpointer userdate)
{
	guint8 ired, iblue, igreen, ialpha;
	
	gnome_color_picker_get_i8(GNOME_COLOR_PICKER(preferences_dialog.edge_colour), &ired, &iblue, &igreen, &ialpha);
	temp_preferences.edge_colour = (igreen<<16) | (iblue<<8) | ired;

}


void on_tab_size_changed(GtkRange *range, gpointer user_data)
{
	temp_preferences.tab_size = (int)(gtk_range_get_adjustment(range)->value);	
}


void on_auto_complete_delay_changed(GtkRange *range, gpointer user_data)
{
	temp_preferences.auto_complete_delay = (int)(gtk_range_get_adjustment(range)->value);	
}


void on_calltip_delay_changed(GtkRange *range, gpointer user_data)
{
	temp_preferences.calltip_delay = (int)(gtk_range_get_adjustment(range)->value);	
}


void on_edge_column_changed(GtkRange *range, gpointer user_data)
{
	temp_preferences.edge_column = (int)(gtk_range_get_adjustment(range)->value);	
}


void on_show_indentation_guides_toggle(GtkToggleButton *togglebutton, gpointer user_data)
{
	temp_preferences.show_indentation_guides = gtk_toggle_button_get_active(togglebutton);	
}


/* Version for gnome_file_entry
void on_php_binary_location_changed (GtkEditable *editable, gpointer user_data)
{
	temp_preferences.php_binary_location = g_strdup(gtk_entry_get_text(GTK_ENTRY(editable)));
}*/


void on_php_binary_location_changed (GtkEntry *entry, gpointer user_data)
{
	temp_preferences.php_binary_location = g_strdup(gtk_entry_get_text(entry));
}

void preferences_dialog_create (void)
{
	GList *comboitems = NULL;
	
	get_current_preferences();

	preferences_dialog.window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (preferences_dialog.window), "Preferences");
	gtk_window_set_modal (GTK_WINDOW (preferences_dialog.window), TRUE);
	gtk_window_set_resizable (GTK_WINDOW (preferences_dialog.window), FALSE);
	gtk_window_set_destroy_with_parent (GTK_WINDOW (preferences_dialog.window), TRUE);

	preferences_dialog.vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (preferences_dialog.vbox1);
	gtk_container_add (GTK_CONTAINER (preferences_dialog.window), preferences_dialog.vbox1);

	preferences_dialog.notebook1 = gtk_notebook_new ();
	gtk_widget_show (preferences_dialog.notebook1);
	gtk_box_pack_start (GTK_BOX (preferences_dialog.vbox1), preferences_dialog.notebook1, TRUE, TRUE, 0);

	preferences_dialog.fixed1 = gtk_fixed_new ();
	gtk_widget_show (preferences_dialog.fixed1);
	gtk_container_add (GTK_CONTAINER (preferences_dialog.notebook1), preferences_dialog.fixed1);

	preferences_dialog.edge_colour = gnome_color_picker_new ();
	gtk_widget_show (preferences_dialog.edge_colour);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed1), preferences_dialog.edge_colour, 160, 96);
	gtk_widget_set_size_request (preferences_dialog.edge_colour, 36, 28);
	gnome_color_picker_set_i8(GNOME_COLOR_PICKER(preferences_dialog.edge_colour), 
		(temp_preferences.edge_colour & 0xff), (temp_preferences.edge_colour & 0xff00) >> 8, (temp_preferences.edge_colour & 0xff0000) >> 16, 0);
	gtk_signal_connect(GTK_OBJECT(GNOME_COLOR_PICKER(preferences_dialog.edge_colour)), "color-set", GTK_SIGNAL_FUNC(on_edge_colour_changed), NULL);	

	preferences_dialog.tab_size = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (temp_preferences.tab_size, 0, 16, 1, 0, 0)));
	gtk_widget_show (preferences_dialog.tab_size);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed1), preferences_dialog.tab_size, 120, 8);
	gtk_widget_set_size_request (preferences_dialog.tab_size, 208, 40);
	gtk_scale_set_digits (GTK_SCALE (preferences_dialog.tab_size), 0);
	gtk_signal_connect (GTK_OBJECT (GTK_HSCALE (preferences_dialog.tab_size)), "value_changed",
                    GTK_SIGNAL_FUNC (on_tab_size_changed), NULL);

	preferences_dialog.edge_column = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (temp_preferences.edge_column, 0, 120, 1, 0, 0)));
	gtk_widget_show (preferences_dialog.edge_column);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed1), preferences_dialog.edge_column, 120, 136);
	gtk_widget_set_size_request (preferences_dialog.edge_column, 208, 40);
	gtk_scale_set_digits (GTK_SCALE (preferences_dialog.edge_column), 0);
	gtk_signal_connect (GTK_OBJECT (GTK_HSCALE (preferences_dialog.edge_column)), "value_changed",
                    GTK_SIGNAL_FUNC (on_edge_column_changed), NULL);

	preferences_dialog.label4 = gtk_label_new ("Tab size");
	gtk_widget_show (preferences_dialog.label4);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed1), preferences_dialog.label4, 8, 24);
	gtk_widget_set_size_request (preferences_dialog.label4, 48, 16);
	gtk_label_set_justify (GTK_LABEL (preferences_dialog.label4), GTK_JUSTIFY_LEFT);

	preferences_dialog.show_indentation_guides = gtk_check_button_new_with_mnemonic ("Show indentation guides");
	gtk_widget_show (preferences_dialog.show_indentation_guides);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed1), preferences_dialog.show_indentation_guides, 8, 64);
	gtk_widget_set_size_request (preferences_dialog.show_indentation_guides, 184, 16);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(preferences_dialog.show_indentation_guides), temp_preferences.show_indentation_guides);
	gtk_signal_connect(GTK_OBJECT(GTK_CHECK_BUTTON(preferences_dialog.show_indentation_guides)), "toggled", GTK_SIGNAL_FUNC(on_show_indentation_guides_toggle), NULL);

	preferences_dialog.label6 = gtk_label_new ("Edge Column");
	gtk_widget_show (preferences_dialog.label6);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed1), preferences_dialog.label6, 8, 155);
	gtk_widget_set_size_request (preferences_dialog.label6, 80, 16);
	gtk_label_set_justify (GTK_LABEL (preferences_dialog.label6), GTK_JUSTIFY_LEFT);

	preferences_dialog.label5 = gtk_label_new ("Right-hand edge colour");
	gtk_widget_show (preferences_dialog.label5);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed1), preferences_dialog.label5, 8, 102);
	gtk_widget_set_size_request (preferences_dialog.label5, 136, 16);
	gtk_label_set_justify (GTK_LABEL (preferences_dialog.label5), GTK_JUSTIFY_LEFT);

	preferences_dialog.label1 = gtk_label_new ("General");
	gtk_widget_show (preferences_dialog.label1);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (preferences_dialog.notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (preferences_dialog.notebook1), 0), preferences_dialog.label1);
	gtk_label_set_justify (GTK_LABEL (preferences_dialog.label1), GTK_JUSTIFY_LEFT);

	preferences_dialog.fixed2 = gtk_fixed_new ();
	gtk_widget_show (preferences_dialog.fixed2);
	gtk_container_add (GTK_CONTAINER (preferences_dialog.notebook1), preferences_dialog.fixed2);

	preferences_dialog.label7 = gtk_label_new ("Highlighting Element");
	gtk_widget_show (preferences_dialog.label7);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed2), preferences_dialog.label7, 8, 16);
	gtk_widget_set_size_request (preferences_dialog.label7, 120, 16);
	gtk_label_set_justify (GTK_LABEL (preferences_dialog.label7), GTK_JUSTIFY_LEFT);

	preferences_dialog.combo1 = gtk_combo_new ();
	gtk_widget_show (preferences_dialog.combo1);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed2), preferences_dialog.combo1, 148, 14);
	gtk_widget_set_size_request (preferences_dialog.combo1, 300, 23);
	gtk_combo_set_use_arrows_always (GTK_COMBO (preferences_dialog.combo1), TRUE);
	gtk_combo_set_popdown_strings (GTK_COMBO (preferences_dialog.combo1), preferences_dialog.combo1_items);
	current_highlighting_element = preferences_dialog.combo1_items->data;
	gtk_signal_connect (GTK_OBJECT (GTK_COMBO (preferences_dialog.combo1)->entry), "changed",
                      GTK_SIGNAL_FUNC (on_combo_entry_changed),
                      NULL);
	g_list_free (preferences_dialog.combo1_items);

	preferences_dialog.combo_entry1 = GTK_COMBO (preferences_dialog.combo1)->entry;
	gtk_widget_show (preferences_dialog.combo_entry1);
	gtk_editable_set_editable (GTK_EDITABLE (preferences_dialog.combo_entry1), FALSE);
	
	preferences_dialog.combo_fontlist = gtk_combo_new ();
	gtk_widget_show(preferences_dialog.combo_fontlist);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed2), preferences_dialog.combo_fontlist, 148, 40);
	gtk_widget_set_size_request (preferences_dialog.combo_fontlist, 180, 23);
	gtk_combo_set_use_arrows_always (GTK_COMBO (preferences_dialog.combo_fontlist), TRUE);
	comboitems = get_font_names();
	gtk_combo_set_popdown_strings (GTK_COMBO (preferences_dialog.combo_fontlist), comboitems);
	gtk_signal_connect (GTK_OBJECT (GTK_COMBO (preferences_dialog.combo_fontlist)->entry), "changed",
                      GTK_SIGNAL_FUNC (on_fontname_entry_changed),
                      NULL);
	g_list_free (comboitems);

	preferences_dialog.combo_fontsize = gtk_combo_new ();
	gtk_widget_show(preferences_dialog.combo_fontsize);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed2), preferences_dialog.combo_fontsize, 348, 40);
	gtk_widget_set_size_request (preferences_dialog.combo_fontsize, 40, 23);
	gtk_combo_set_use_arrows_always (GTK_COMBO (preferences_dialog.combo_fontsize), TRUE);
	comboitems = get_font_sizes();
	gtk_combo_set_popdown_strings (GTK_COMBO (preferences_dialog.combo_fontsize), comboitems);
	gtk_signal_connect (GTK_OBJECT (GTK_COMBO (preferences_dialog.combo_fontsize)->entry), "changed",
                      GTK_SIGNAL_FUNC (on_fontsize_entry_changed),
                      NULL);
	g_list_free (comboitems);

	preferences_dialog.bold_button = gtk_toggle_button_new_with_label("Bold");
	gtk_widget_show(preferences_dialog.bold_button);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed2), preferences_dialog.bold_button, 400, 40);
	gtk_widget_set_size_request (preferences_dialog.bold_button, 40, 23);
	gtk_signal_connect(GTK_OBJECT(GTK_TOGGLE_BUTTON(preferences_dialog.bold_button)), "toggled", GTK_SIGNAL_FUNC(on_bold_toggle), NULL);

	preferences_dialog.italic_button = gtk_toggle_button_new_with_label("Italic");
	gtk_widget_show(preferences_dialog.italic_button);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed2), preferences_dialog.italic_button, 450, 40);
	gtk_widget_set_size_request (preferences_dialog.italic_button, 40, 23);
	gtk_signal_connect(GTK_OBJECT(GTK_TOGGLE_BUTTON(preferences_dialog.italic_button)), "toggled", GTK_SIGNAL_FUNC(on_italic_toggle), NULL);
	



	preferences_dialog.label8 = gtk_label_new ("Font");
	gtk_widget_show (preferences_dialog.label8);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed2), preferences_dialog.label8, 8, 48);
	gtk_widget_set_size_request (preferences_dialog.label8, 36, 16);
	gtk_label_set_justify (GTK_LABEL (preferences_dialog.label8), GTK_JUSTIFY_LEFT);

	preferences_dialog.label9 = gtk_label_new ("Foreground (text) colour");
	gtk_widget_show (preferences_dialog.label9);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed2), preferences_dialog.label9, 8, 88);
	gtk_widget_set_size_request (preferences_dialog.label9, 152, 16);
	gtk_label_set_justify (GTK_LABEL (preferences_dialog.label9), GTK_JUSTIFY_LEFT);

	preferences_dialog.label10 = gtk_label_new ("Background (paper) colour");
	gtk_widget_show (preferences_dialog.label10);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed2), preferences_dialog.label10, 16, 128);
	gtk_widget_set_size_request (preferences_dialog.label10, 160, 16);
	gtk_label_set_justify (GTK_LABEL (preferences_dialog.label10), GTK_JUSTIFY_LEFT);

	preferences_dialog.fore_color_picker = gnome_color_picker_new ();
	gtk_widget_show (preferences_dialog.fore_color_picker);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed2), preferences_dialog.fore_color_picker, 208, 80);
	gtk_widget_set_size_request (preferences_dialog.fore_color_picker, 36, 28);
	gtk_signal_connect(GTK_OBJECT(GNOME_COLOR_PICKER(preferences_dialog.fore_color_picker)), "color-set", GTK_SIGNAL_FUNC(on_fore_changed), NULL);	

	preferences_dialog.back_color_picker = gnome_color_picker_new ();
	gtk_widget_show (preferences_dialog.back_color_picker);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed2), preferences_dialog.back_color_picker, 208, 120);
	gtk_widget_set_size_request (preferences_dialog.back_color_picker, 36, 28);
	gtk_signal_connect(GTK_OBJECT(GNOME_COLOR_PICKER(preferences_dialog.back_color_picker)), "color-set", GTK_SIGNAL_FUNC(on_back_changed), NULL);	

	preferences_dialog.label2 = gtk_label_new ("Syntax Highlighting");
	gtk_widget_show (preferences_dialog.label2);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (preferences_dialog.notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (preferences_dialog.notebook1), 1), preferences_dialog.label2);
	gtk_label_set_justify (GTK_LABEL (preferences_dialog.label2), GTK_JUSTIFY_LEFT);

	preferences_dialog.fixed3 = gtk_fixed_new ();
	gtk_widget_show (preferences_dialog.fixed3);
	gtk_container_add (GTK_CONTAINER (preferences_dialog.notebook1), preferences_dialog.fixed3);

	preferences_dialog.calltip_delay = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (temp_preferences.calltip_delay, 0, 2500, 0, 0, 0)));
	gtk_widget_show (preferences_dialog.calltip_delay);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed3), preferences_dialog.calltip_delay, 176, 120);
	gtk_widget_set_size_request (preferences_dialog.calltip_delay, 152, 40);
	gtk_scale_set_digits (GTK_SCALE (preferences_dialog.calltip_delay), 0);
	gtk_signal_connect (GTK_OBJECT (GTK_HSCALE (preferences_dialog.calltip_delay)), "value_changed",
                    GTK_SIGNAL_FUNC (on_calltip_delay_changed), NULL);

	preferences_dialog.auto_complete_delay = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (temp_preferences.auto_complete_delay, 0, 2500, 0, 0, 0)));
	gtk_widget_show (preferences_dialog.auto_complete_delay);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed3), preferences_dialog.auto_complete_delay, 176, 64);
	gtk_widget_set_size_request (preferences_dialog.auto_complete_delay, 152, 40);
	gtk_scale_set_digits (GTK_SCALE (preferences_dialog.auto_complete_delay), 0);
	gtk_signal_connect (GTK_OBJECT (GTK_HSCALE (preferences_dialog.auto_complete_delay)), "value_changed",
                    GTK_SIGNAL_FUNC (on_auto_complete_delay_changed), NULL);


	preferences_dialog.label12 = gtk_label_new ("Drop-down functions delay");
	gtk_widget_show (preferences_dialog.label12);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed3), preferences_dialog.label12, 8, 136);
	gtk_widget_set_size_request (preferences_dialog.label12, 160, 16);
	gtk_label_set_justify (GTK_LABEL (preferences_dialog.label12), GTK_JUSTIFY_LEFT);

	preferences_dialog.label11 = gtk_label_new ("Calltip (hint) function delay");
	gtk_widget_show (preferences_dialog.label11);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed3), preferences_dialog.label11, 8, 80);
	gtk_widget_set_size_request (preferences_dialog.label11, 160, 16);
	gtk_label_set_justify (GTK_LABEL (preferences_dialog.label11), GTK_JUSTIFY_LEFT);

	preferences_dialog.label13 = gtk_label_new ("Folder containing PHP binary");
	gtk_widget_show (preferences_dialog.label13);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed3), preferences_dialog.label13, 8, 24);
	gtk_widget_set_size_request (preferences_dialog.label13, 170, 16);
	gtk_label_set_justify (GTK_LABEL (preferences_dialog.label13), GTK_JUSTIFY_LEFT);

	/*preferences_dialog.php_binary_location = gnome_file_entry_new (NULL, "PHP Command Line Binary");
	gtk_widget_show (preferences_dialog.php_binary_location);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed3), preferences_dialog.php_binary_location, 136, 19);
	gtk_widget_set_size_request (preferences_dialog.php_binary_location, 184, 27);
  	g_signal_connect ((gpointer) preferences_dialog.php_binary_location, "changed",
                    G_CALLBACK (on_php_binary_location_changed),
                    NULL);	
	*/
	preferences_dialog.php_binary_location = gtk_entry_new();
	gtk_widget_show (preferences_dialog.php_binary_location);
	gtk_fixed_put (GTK_FIXED (preferences_dialog.fixed3), preferences_dialog.php_binary_location, 180, 19);
	gtk_widget_set_size_request (preferences_dialog.php_binary_location, 184, 27);
	gtk_entry_set_text(GTK_ENTRY(preferences_dialog.php_binary_location), preferences.php_binary_location);
	gtk_signal_connect (GTK_OBJECT (GTK_ENTRY (preferences_dialog.php_binary_location)), "changed",
                    GTK_SIGNAL_FUNC (on_php_binary_location_changed), NULL);
					

	preferences_dialog.label3 = gtk_label_new ("PHP Features");
	gtk_widget_show (preferences_dialog.label3);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (preferences_dialog.notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (preferences_dialog.notebook1), 2), preferences_dialog.label3);
	gtk_label_set_justify (GTK_LABEL (preferences_dialog.label3), GTK_JUSTIFY_LEFT);

	preferences_dialog.hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (preferences_dialog.hbox1);
	gtk_box_pack_start (GTK_BOX (preferences_dialog.vbox1), preferences_dialog.hbox1, FALSE, FALSE, 0);

	preferences_dialog.button1 = gtk_button_new_from_stock ("gtk-cancel");
	gtk_widget_show (preferences_dialog.button1);
	gtk_box_pack_end (GTK_BOX (preferences_dialog.hbox1), preferences_dialog.button1, FALSE, FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (preferences_dialog.button1), 4);

	preferences_dialog.button2 = gtk_button_new_from_stock ("gtk-ok");
	gtk_widget_show (preferences_dialog.button2);
	gtk_box_pack_end (GTK_BOX (preferences_dialog.hbox1), preferences_dialog.button2, FALSE, FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (preferences_dialog.button2), 4);

	gtk_signal_connect (GTK_OBJECT (preferences_dialog.button2),
	                    "clicked", GTK_SIGNAL_FUNC (ok_clicked), NULL);

	// Hide the dialog box when the user clicks the cancel_button
	gtk_signal_connect_object (GTK_OBJECT (preferences_dialog.button1),
	                           "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), (gpointer) preferences_dialog.window);

	gtk_signal_connect(GTK_OBJECT(preferences_dialog.window), "destroy", GTK_SIGNAL_FUNC(preferences_window_destroyed), NULL);

	get_current_highlighting_settings(current_highlighting_element);
}
